-- Title: CRC test project
-- Copyright notice:  (C) 2017 www.dlsconsulting.de
-- Use this file for test and simulation only
-- Note, that no warranty on completeness or correctness can be taken over for this source code

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library work;
use work.crc_definitions.all;

entity CRCT_top is 
   port(
   ------------------------------------------------------------------------------------------               
   nRESET            : in std_logic;                                                -- reset input  
   CLK               : in std_logic;                                                -- clock input
   DBYTE_IN          : in std_logic_vector(7 DOWNTO 0);                             -- 8bit data input
   STB               : in std_logic;                                                -- data strobe input
   FINISH            : in std_logic;                                                -- CRC finish input
   CRC_OUT           : out std_logic_vector(15 DOWNTO 0)                            -- 16bit CRC output
   );
end CRCT_top;

architecture CRCT_arch of CRCT_top is 
   signal crc16      : std_logic_vector(15 downto 0);                               -- CRC variable
   
-- finite state machine definitions:
   type state_type is (
         init,
         idle,
         c_crc,
         wfi,
         fin_crc,
         stop
   );
   signal current_state : state_type;

begin
   CRC_OUT <= crc16;
   
   ------------------------------------------------------------------------------------------               
   -- CRCT finite state machine
   ------------------------------------------------------------------------------------------               
   state_actions: process(CLK, nRESET)
   ------------------------------------------------------------------------------------------               
   begin
      if (nRESET = '0') then
         current_state <= init;
         crc16 <= x"0000";
      elsif rising_edge(CLK) then
         case current_state is
            when init =>
               crc16 <= CRC16_Ini;                    -- set crc initial value
               current_state <= idle;
            when idle =>
               if STB = '1' then
                  current_state <= c_crc;
               elsif FINISH = '1' then
                  current_state <= fin_crc;
               else
                  current_state <= idle;
               end if;
            when c_crc =>
               crc16 <= NewCRC(crc16, DBYTE_IN);      -- parallel crc calculation
               current_state <= wfi;
            when wfi =>
               if STB = '1' then
                  current_state <= wfi;
               else
                  current_state <= idle;
               end if;
            when fin_crc =>
               crc16 <= crc16 xor CRC16_Fin;          -- final xor with the defined value
               current_state <= stop;
            when stop =>
               current_state <= stop;
            when others =>
               current_state <= stop; 
         end case;
      end if;
   end process state_actions;
 
end CRCT_arch;


